local _, T = ...
local L = T.L

local Farmhand = CreateFrame("frame","Farmhand",UIParent)
Farmhand:RegisterEvent("PLAYER_LOGIN")
Farmhand:RegisterEvent("PLAYER_ENTERING_WORLD")
local function Farmhand_EventHandler(self, event, ...)
	if event == "PLAYER_LOGIN" then
		self:Initialize()
	elseif event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("BAG_UPDATE")
		self:RegisterEvent("ZONE_CHANGED")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("ZONE_CHANGED_INDOORS")
	elseif event == "ZONE_CHANGED" or 
	       event == "ZONE_CHANGED_NEW_AREA" or 
		   event == "ZONE_CHANGED_INDOORS" then
		self:UpdateLayout()
	elseif event == "BAG_UPDATE" then
		self:UpdateLayout()
	elseif event == "PLAYER_REGEN_ENABLED" then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		if self.Initialized then
			self:UpdateLayout()
		else
			self:Initialize()
		end	
	end
end
Farmhand:SetScript("OnEvent",Farmhand_EventHandler)
Farmhand:SetMovable(true)

Farmhand.tools = CreateFrame("Frame","FarmhandTools",Farmhand)
Farmhand.tools:SetScale(1.5)
Farmhand.tools.items = {
	{id=89815}, --Master Plow
	{id=89880}, --Dented Shovel
	{id=80513}, --Vintage Bug Sprayer
	{id=79104}, --Rusty Watering Can
	{id="scanner"}, --Plant Scanner
}

Farmhand.seeds = CreateFrame("Frame","FarmhandSeeds",Farmhand)
Farmhand.seeds:SetScale(1)
Farmhand.seeds.items = {
	{id=79102}, --Green Cabbage Seeds
	{id=89328}, --Jade Squash Seeds
	{id=80590}, --Juicycrunch Carrot Seeds
	{id=80592}, --Mogu Pumpkin Seeds
	{id=80594}, --Pink Turnip Seeds
	{id=80593}, --Red Blossom Leek Seeds
	{id=80591}, --Scallion Seeds
	{id=89329}, --Striped Melon Seeds
	{id=80595}, --White Turnip Seeds
	{id=89326}, --Witchberry Seeds
	{id=85219}, --Ominous Seed
	{id=85216}, --Enigma Seed
	{id=85217}, --Magebulb Seed
	{id=89202}, --Raptorleaf Seed
	{id=85215}, --Snakeroot Seed
	{id=89233}, --Songbell Seed
	{id=89197}, --Windshear Cactus Seed
	{id=91860}, --Stormwind Portal Shard
	{id=91864}, --Ironforge Portal Shard
	{id=91865}, --Darnassus Portal Shard
	{id=91866}, --Exodar Portal Shard
	{id=91850}, --Orgrimmar Portal Shard
	{id=91862}, --Undercity Portal Shard
	{id=91861}, --Thunder Bluff Portal Shard
	{id=91863}, --Silvermoon Portal Shard
}

local lock = CreateFrame("CheckButton","FarmhandToolLock",FarmhandTools)
lock:SetBackdrop({bgFile="Interface\\Buttons\\LockButton-Border"})
lock:SetNormalTexture("Interface\\Buttons\\LockButton-Unlocked-Up")
lock:SetPushedTexture("Interface\\Buttons\\LockButton-Unlocked-Down")
lock:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
lock:SetCheckedTexture("Interface\\Buttons\\LockButton-Locked-Up")
lock:SetSize(16,16)
lock:SetPoint("Right",FarmhandTools,"Left",0,0)
RegisterStateDriver(lock,"visibility","[mod:shift] show; hide")
lock:SetScript("OnClick",function(self)
	if self:GetChecked() == 1 then
		FarmhandData.ToolsLocked = true
		print(L["Tools are now |cFF00FF00Locked|r and |cFF00FF00will not|r be dropped when you leave Halfhill."])
	else
		FarmhandData.ToolsLocked = false
		print(L["Tools are now |cFFFF0000Unlocked|r and |cFFFF0000will|r be dropped when you leave Halfhill."])
	end
end)

Farmhand.lock = lock

function Farmhand:Initialize()
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	if FarmhandData == nil then
		FarmhandData = {
			X = 0,
			Y = -250,
			ToolsLocked = false,
		}
	end

	self.lock:SetChecked(FarmhandData.ToolsLocked == true)
	
	self:CreateButtons(self.tools)
	self:CreateButtons(self.seeds)
	self:UpdateLayout()
	self.Initialized = true
end

function Farmhand:CreateButtons(bar)
	local msq, msqTools
	if LibStub then
		msq = LibStub("Masque",true)
	end
	if msq then
		msqTools = msq:Group("Farmhand")
	end
	for i,item in ipairs(bar.items) do
	
		if item.id == "scanner" then
			item.btn = CreateFrame("Button", "FarmhandScanButton", FarmhandTools, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)
			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetTexture("Interface\\AddOns\\Farmhand\\RadarIcon.tga")
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()

			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")

			item.btn:RegisterForClicks("AnyUp")
			item.btn:SetAttribute("type", "macro")
			item.btn:SetAttribute("macrotext", L["CropScannerMacro"])
			
			item.btn:SetScript("PostClick",function(self,button,down)
				if down or (button ~= "LeftButton" and button ~= "RightButton") then return end
				if UnitExists("target") then 
					SetRaidTarget("target",8)
				else 
					RaidNotice_AddMessage(RaidBossEmoteFrame,L["The crops are looking good!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
				end
			end)
			
		else
			local icon = select(10,GetItemInfo(item.id))

			item.btn = CreateFrame("Button", bar:GetName().."Button"..i, bar, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)

			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetTexture(icon)
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()

			item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
			item.btn.count:SetJustifyH("RIGHT")
			item.btn.count:SetPoint("BottomRight",-2,2)

			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
			
			item.btn:RegisterForClicks("AnyUp")
			item.btn:SetAttribute("type", "item")
			item.btn:SetAttribute("item1", "item:"..item.id)
		end
		
		item.btn:SetScript("OnEnter",Farmhand.ButtonOnEnter)
		item.btn:SetScript("OnLeave",function() GameTooltip_Hide() end)
		item.btn:SetScript("OnMouseDown",Farmhand.ButtonOnMouseDown)
		item.btn:SetScript("OnMouseUp",Farmhand.ButtonOnMouseUp)
		item.btn:SetScript("OnHide",Farmhand.ButtonOnHide)
		if msqTools then
			msqTools:AddButton(item.btn)
		end

	end
end

function Farmhand:ButtonOnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	if self:GetAttribute("item1") then
		local _, link = GetItemInfo(self:GetAttribute("item1"))
		GameTooltip:SetHyperlink(link)
	else
		GameTooltip:ClearLines()
		GameTooltip:AddLine(L["Crop Scanner"],0,255,0,false)
		GameTooltip:AddLine(L["Scan your farm for crops or soil that need attention."],255,255,255,true)
	end
	GameTooltip:Show()
end

function Farmhand:ButtonOnMouseDown(button)
	if IsShiftKeyDown() and button == "LeftButton" and not Farmhand.isMoving then
		Farmhand:StartMoving()
		Farmhand.isMoving = true
	elseif IsShiftKeyDown() and button == "RightButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		FarmhandData.X, FarmhandData.Y = 0, -250
		Farmhand:SetPoint("Center",UIParent,"Center",0,-250)
		Farmhand.isMoving = false
	end
end

function Farmhand:ButtonOnMouseUp(button)
	if button == "LeftButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		_,_,_,FarmhandData.X,FarmhandData.Y = Farmhand:GetPoint(1)
		Farmhand.isMoving = false
   end
end

function Farmhand:ButtonOnHide()
	if ( Farmhand.isMoving ) then
		Farmhand:StopMovingOrSizing()
		_,_,_,FarmhandData.X,FarmhandData.Y = Farmhand:GetPoint(1)
		Farmhand.isMoving = false
	end
end

function Farmhand:UpdateBar(bar)
	local last
	local shown = 0
	local ButtonSpacing = 6

	for _,item in ipairs(bar.items) do
		local ItemCount = GetItemCount(item.id)
		if ItemCount > 0 or item.id == "scanner" then
			if shown % 8 == 0 then
				local row = math.floor(shown/8)
				item.btn:SetPoint("TopLeft",bar,"TopLeft", ButtonSpacing/2, -(ButtonSpacing * .5 ) - row*item.btn:GetHeight() - row*ButtonSpacing)
				last = item.btn
			else
				item.btn:SetPoint("TopLeft",last,"TopRight",ButtonSpacing,0)
				last = item.btn
			end
			if IsConsumableItem(item.id) then
				local count = _G[item.btn:GetName().."Count"]
				if ItemCount > 999 then
					count:SetText("*")
				else
					count:SetText(ItemCount)
				end
			end
			local bag, slot = self:FindItemInBags(item.id)
			if bag and slot then
				item.btn:SetAttribute("item2",bag.." "..slot)
			end
			item.btn:Show()
			shown = shown + 1
		else
			item.btn:Hide()
		end
	end
	
	if last ~= nil then
		bar:SetWidth(math.min(8,shown) * (last:GetWidth() + ButtonSpacing))
		bar:SetHeight(math.ceil(shown/8) * (last:GetHeight() + ButtonSpacing))
		bar:Show()
	else
		bar:Hide()
	end
end

function Farmhand:UpdateLayout()

	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	local Zone, SubZone = GetZoneText(), GetSubZoneText()

	if Zone == "" and SubZone == "" then 
		return
	end

	if self.InHalfhill and SubZone ~= L["Sunsong Ranch"] and SubZone ~= L["The Halfhill Market"] and SubZone ~= L["Halfhill"] and Zone ~= L["Halfhill"] then
		if FarmhandData.ToolsLocked == true then
			print(L["Leaving Halfhill."].." "..L["Tools are Locked."])
		else
			ClearCursor()
			for i,item in ipairs(self.tools.items) do
				local bag, slot = self:FindItemInBags(item.id)
				if bag and slot then
					PickupContainerItem(bag,slot)
					if CursorHasItem() then
						local _, id, link = GetCursorInfo()
						if id == item.id then
							print(L["Leaving Halfhill."].." "..L["Dropping"].." "..link..".")
							DeleteCursorItem()
						end
					end
				end
			end
		end
	end

	self.InHalfhill = SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] or SubZone == L["Halfhill"] or Zone == L["Halfhill"]
	
	if SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] then 
		self:UpdateBar(self.tools)
		self:UpdateBar(self.seeds)
			
		local th = (self.tools:IsShown() and self.tools:GetHeight()*self.tools:GetScale() or 0)
		local sh = (self.seeds:IsShown() and self.seeds:GetHeight()*self.seeds:GetScale() or 0)
		self:SetHeight(24 + th + sh)
		
		local tw = (self.tools:IsShown() and self.tools:GetWidth()*self.tools:GetScale() or 0)
		local sw = (self.seeds:IsShown() and self.seeds:GetWidth()*self.seeds:GetScale() or 0)
		self:SetWidth(24 + math.max(tw,sw))
		
		self.seeds:SetPoint("Top",self,"Top",0,-15)
		self.tools:SetPoint("Bottom",self,"Bottom",0,7)
		self.tools:SetPoint("Bottom",self,"Bottom",0,7)
		self:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
		self:Show()
	else
		self:Hide()
	end

end

function Farmhand:FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 0, numSlots do
				for slot = 1, numSlots do
					if item == GetContainerItemID(container, slot) then
						return container, slot
					end
				end
			end
		end
	end
end
